/* xtoa_fast.h
 This is a temporary file to debug` xtoa_fast' family. Its contents
 would be carried to `stdlib.h', I think.
 2005-02-06	Begin.
 2005-02-16	Last modif.
*/
#ifndef	_XTOA_FAST_H_
#define _XTOA_FAST_H_

#ifndef	__ASSEMBLER__

#include <stddef.h>	/* for 'size_t'	*/

/**
   \brief Convert an integer to a string.

   The function itoa_fast() converts the integer value from \c val into
   an ASCII representation that will be stored under \c s.  The caller
   is responsible for providing sufficient storage in \c s.

   \note The minimal size of the buffer \c s depends on the choice of
   radix (\c base). For example, if the radix is 2 (binary), you need to
   supply a buffer with a minimal length of 8 * sizeof (int) + 1
   characters, i.e. one character for each bit plus one for the string
   terminator. Using a larger radix will require a smaller minimal
   buffer size.

   \warning If the buffer is too small, you risk a buffer overflow.

   Conversion is done using the \c base, which may be a 2, 8, 10 or 16.
   Adding to \c base corresponding tags, it is possible to establish a
   mode of a prefix (XTOA_PREFIX) and/or the big letters (XTOA_UPPER),
   for example:

        char s[10];
        itoa_fast (80, s, XTOA_PREFIX | 16);   -->  "0x50"

   Notes:
      - similarly 'itoa', numbers not on the basis 10 are deduced
      as unsigned,
      - similarly 'printf', to zero the prefix is not added,
      - in case of inadmissible value of 'base' result is the empty line,
      - XTOA_PREFIX is meaning for bases 8 and 16,
      - XTOA_UPPER is meaning for base 16.

   The itoa_fast() function returns the pointer passed as \c s.
 */
char * itoa_fast (int val, char *s, int base);
char * utoa_fast (unsigned val, char *s, int base);
char * ltoa_fast (long val, char *s, int base);
char * ultoa_fast (unsigned long val, char *s, int base);

/**
    \brief Convert an integer to a string with field alignment.
    
    It happens, the limited resources of the microcontroller do not
    allow to take advantage 'printf'. Then beautiful tables of numbers
    can be received with the help of the function itoa_width().

    In case of the decimal basis the empty seat is filled by spaces (to
    the left of number). For the other bases -- by zero. If the output
    of a prefix is set, it remains at the left edge. Examples:

        itoa_width (-1, s, 10, 6)                 -->  "    -1"
        itoa_width (255, s, XTOA_PREFIX | 16, 6)  -->  "0x00ff"

    Notes:
      - similarly 'printf', there is no truncation of result for
      small 'width'.
 */
char * itoa_width (int val, char *s, int base, size_t width);
char * utoa_width (unsigned val, char *s, int base, size_t width);
char * ltoa_width (long val, char *s, int base, size_t width);
char * ultoa_width (unsigned long val, char *s, int base, size_t width);

/* Internal function for use from `printf'.	*/
char * __ultoa_invert (unsigned long val, char *s, int base);

#endif	/* ifndef __ASSEMBLER__ */

/* Next flags are to use with `base'. Unused fields are reserved.	*/
#define XTOA_PREFIX	0x0100	/* put prefix for octal or hex	*/
#define XTOA_UPPER	0x0200	/* use upper case letters	*/

#endif	/* _XTOA_FAST_H_ */
