/* (i/u)toa_width() test with table of patterns. Base is 16.	*/
#include <stdlib.h>
#include <string.h>
#include "xtoa_fast.h"

#define SIZEMAX     9

/* Compare with pattern.	*/
static int
cmp (unsigned int val, int flags, size_t width, const char * pattern)
{
    char s[SIZEMAX];
    char *p;
    
    p = itoa_width (val, s, flags | 16, width);
    if (p != s) return 1;
    if (strcmp(s, pattern)) return 1;

    p = utoa_width (val, s, flags | 16, width);
    if (p != s) return 1;
    if (strcmp(s, pattern)) return 1;

    return 0;
}

int main ()
{
    static struct t_s {
	int val;
	int flags;
	unsigned char width;
	char s[SIZEMAX];
    } t[] = {

	{ 0, 0, 1,	"0" },
	{ 0, 0, 2,	"00" },
	{ 0, 0, 8,	"00000000" },
	
	{ 1, 0, 1,	"1" },
	{ 1, 0, 2,	"01" },
	{ 1, 0, 8,	"00000001" },
	
	{ -1, 0, 3,	"ffff" },
	{ -1, 0, 4,	"ffff" },
	{ -1, 0, 5,	"0ffff" },
	{ -1, 0, 8,	"0000ffff" },
	
	{ 0x10, 0, 2,		"10" },
	{ 0x10, 0, 3,		"010" },
	{ 0x100, 0, 3,		"100" },
	{ 0x100, 0, 4,		"0100" },
	{ 0x1000, 0, 4,		"1000" },
	{ 0x1000, 0, 5,		"01000" },
	
	{ 0, XTOA_PREFIX, 0,	"0" },
	{ 0, XTOA_PREFIX, 1,	"0" },
	{ 0, XTOA_PREFIX, 2,	"0" },
	{ 0, XTOA_PREFIX, 3,	"0x0" },
	
	{ 1, XTOA_PREFIX, 1,	"0x1" },
	{ 1, XTOA_PREFIX, 2,	"0x1" },
	{ 1, XTOA_PREFIX, 3,	"0x1" },
	{ 1, XTOA_PREFIX, 4,	"0x01" },
	
	{ -1, XTOA_PREFIX, 5,	"0xffff" },
	{ -1, XTOA_PREFIX, 6,	"0xffff" },
	{ -1, XTOA_PREFIX, 7,	"0x0ffff" },
	
	{ 0xabcd, XTOA_UPPER, 3,	"ABCD" },
	{ 0xabcd, XTOA_UPPER, 4,	"ABCD" },
	{ 0xabcd, XTOA_UPPER, 5,	"0ABCD" },
	{ 0xabcd, XTOA_UPPER | XTOA_PREFIX, 5,	"0XABCD" },
	{ 0xabcd, XTOA_UPPER | XTOA_PREFIX, 6,	"0XABCD" },
	{ 0xabcd, XTOA_UPPER | XTOA_PREFIX, 7,	"0X0ABCD" },
    };
    struct t_s *p;
    int i;
    
    i = 0;
    for (p = t; p != t + sizeof(t)/sizeof(t[0]); p++) {
	i += 1;
	if (cmp(p->val, p->flags, p->width, p->s))
	    exit(i);
    }
    
    return 0;
}
