/* This test include all possible conversion (long, fast) together.
   'strtol/strtoul' is used for result verification.	*/
#include <ctype.h>
#include <errno.h>
#include <stdlib.h>
#include <string.h>
#include "xtoa_fast.h"

void run (unsigned long val)
{
    char s[33];		/* size for -1 in binary	*/
    char t[33];
    char *p;

    p = ltoa_fast (val, s, 10);
    if (p != s) exit(__LINE__);
    if (val >= 0x80000000u) {
	if (s[0] != '-') exit(__LINE__);
	if (s[1] == '0') exit(__LINE__);
    } else {
	if (!isdigit(s[0])) exit(__LINE__);
	if (val) {
	    if (s[0] == '0') exit(__LINE__);
	} else {
	    if (strlen(s) != 1) exit(__LINE__);
	}
    }
    errno = 0;
    if ((unsigned long)strtol(s, &p, 10) != val) exit(__LINE__);
    if (errno || p == s || *p)		/* error or invalid character	*/
	exit(__LINE__);

    p = ultoa_fast (val, s, 10);
    if (p != s) exit(__LINE__);
    if (!isdigit(s[0])) exit(__LINE__);
    if (val) {
	if (s[0] == '0') exit(__LINE__);
    } else {
	if (strlen(s) != 1) exit(__LINE__);
    }
    if (strtoul(s, &p, 10) != val) exit(__LINE__);
    if (p == s || *p) exit(__LINE__);	/* error or invalid character	*/
    
    p = ltoa_fast (val, s, 2);
    if (p != s) exit(__LINE__);
    p = ultoa_fast (val, t, 2);
    if (p != t) exit(__LINE__);
    if (strcmp(s, t)) exit(__LINE__);
    if (val) {
	if (s[0] != '1') exit(__LINE__);
    } else {
	if (strlen(s) != 1) exit(__LINE__);
    }
    if (strtoul(s, &p, 2) != val) exit(__LINE__);
    if (p == s || *p) exit(__LINE__);	/* error or invalid character	*/
    
    p = ltoa_fast (val, s, 8);
    if (p != s) exit(__LINE__);
    p = ultoa_fast (val, t, 8);
    if (p != t) exit(__LINE__);
    if (strcmp(s, t)) exit(__LINE__);
    if (val) {
	if (s[0] == '0') exit(__LINE__);
    } else {
	if (strlen(s) != 1) exit(__LINE__);
    }
    if (strtoul(s, &p, 8) != val) exit(__LINE__);
    if (p == s || *p) exit(__LINE__);	/* error or invalid character	*/
    
    p = ltoa_fast (val, s, 8 | XTOA_PREFIX);
    if (p != s) exit(__LINE__);
    p = ultoa_fast (val, t, 8 | XTOA_PREFIX);
    if (p != t) exit(__LINE__);
    if (strcmp(s, t)) exit(__LINE__);
    if (s[0] != '0') exit(__LINE__);
    if (val) {
	if (strlen(s) < 2) exit(__LINE__);
	if (s[1] == '0') exit(__LINE__);
    } else {
	if (strlen(s) != 1) exit(__LINE__);
    }
    if (strtoul(s, &p, 8) != val) exit(__LINE__);
    if (p == s || *p) exit(__LINE__);	/* error or invalid character	*/

    p = ltoa_fast (val, s, 16);
    if (p != s) exit(__LINE__);
    p = ultoa_fast (val, t, 16);
    if (p != t) exit(__LINE__);
    if (strcmp(s, t)) exit(__LINE__);
    if (val) {
	if (s[0] == '0') exit(__LINE__);
    } else {
	if (strlen(s) != 1) exit(__LINE__);
    }
    for (p= s; *p; p++)
	if (isupper(*p)) exit(__LINE__);
    if (strtoul(s, &p, 16) != val) exit(__LINE__);
    if (p == s || *p) exit(__LINE__);	/* error or invalid character	*/

    p = ltoa_fast (val, s, 16 | XTOA_UPPER);
    if (p != s) exit(__LINE__);
    p = ultoa_fast (val, t, 16 | XTOA_UPPER);
    if (p != t) exit(__LINE__);
    if (strcmp(s, t)) exit(__LINE__);
    if (val) {
	if (s[0] == '0') exit(__LINE__);
    } else {
	if (strlen(s) != 1) exit(__LINE__);
    }
    for (p= s; *p; p++)
	if (islower(*p)) exit(__LINE__);
    if (strtoul(s, &p, 16) != val) exit(__LINE__);
    if (p == s || *p) exit(__LINE__);	/* error or invalid character	*/

    p = ltoa_fast (val, s, 16 | XTOA_PREFIX);
    if (p != s) exit(__LINE__);
    p = ultoa_fast (val, t, 16 | XTOA_PREFIX);
    if (p != t) exit(__LINE__);
    if (strcmp(s, t)) exit(__LINE__);
    if (val) {
	if (strlen(s) < 3) exit(__LINE__);
	if (strncmp(s, "0x", 2)) exit(__LINE__);
    } else {
	if (strlen(s) != 1) exit(__LINE__);
    }
    for (p= s; *p; p++)
	if (isupper(*p)) exit(__LINE__);
    if (strtoul(s, &p, 16) != val) exit(__LINE__);
    if (p == s || *p) exit(__LINE__);	/* error or invalid character	*/

    p = ltoa_fast (val, s, 16 | XTOA_PREFIX | XTOA_UPPER);
    if (p != s) exit(__LINE__);
    p = ultoa_fast (val, t, 16 | XTOA_PREFIX | XTOA_UPPER);
    if (p != t) exit(__LINE__);
    if (strcmp(s, t)) exit(__LINE__);
    if (val) {
	if (strlen(s) < 3) exit(__LINE__);
	if (strncmp(s, "0X", 2)) exit(__LINE__);
    } else {
	if (strlen(s) != 1) exit(__LINE__);
    }
    for (p= s; *p; p++)
	if (islower(*p)) exit(__LINE__);
    if (strtoul(s, &p, 16) != val) exit(__LINE__);
    if (p == s || *p) exit(__LINE__);	/* error or invalid character	*/
}

int main ()
{
    unsigned long val;
    int i;

    for (val= 0; val <= 256; val++)
	run(val);

    for (val= 0xffffffff; val >= 0xffffff00; val--)
	run(val);
	
    /* one bit is set	*/
    for (val= 1; val; val <<= 1)
	run(val);

    /* one bit is clean	*/
    for (val= ~1L; val != ~0Lu; val= (val << 1) + 1)
	run(val);

    /* random	*/
    for (i= 500; i; i--) {
	val= (val << 15) ^ rand();
	run(val);
    }
    
    return 0;
}
