#! /bin/bash

# Script to run a set of tests.
# Installed program are needed: avr-gcc, simulavr and Avr-libc.
# Possible arguments are: a list of test's (C sources).

srcdir=../src
incdir=../include
test_list=${*:-"t_*.c"}

set -e

avr_gcc=${AVRGCC:-"avr-gcc"}

core=core_avr_dump.core

set_src_list ()
{
    host_list=
    only_avr=
    case $1 in
        *_itoa* )
            src_list="$srcdir/itoa_fast.S"
            only_avr=1
            ;;
        *_ltoa* )
            src_list="$srcdir/ltoa_fast.S $srcdir/ultoa_invert.S"
            only_avr=1
            ;;
        *_sprf_*)
            src_list="$srcdir/vfprintf.c $srcdir/ultoa_invert.S"
            ;;
        *)
            src_list=
            ;;
    esac
}

simulate ()
{
    echo "-------- simulate:  $*"
    bname=`basename $1 .c`
    shift
    mcu=$1
    shift
    $avr_gcc $cflags -mmcu=$mcu $* -o $bname.elf $bname.c $src_list
    exit_addr=0x`avr-nm $bname.elf | grep __stop_program | cut -f1 -d' '`
    avr-objcopy -O binary $bname.elf $bname.bin
    simulavr -d $mcu -B $exit_addr -C $bname.bin >/dev/null
    if ! grep -q r24=00 $core || ! grep -q r25=00 $core
    then
        echo >&2 "*** Simulate: exit with non-zero code"
        exit 1
    fi
    rm -f $bname.{elf,bin} $core
}

run_simulate ()
{
    for mcu in at90s8515 atmega8 atmega16 ; do
        for opt in "-Os" ; do
            simulate $1 $mcu $opt
        done
    done
}

run_host ()
{
    echo "-------- test of test: $1"
    bname=`basename $1 .c`
    cc $cflags -D TEST_OF_TEST -O2 -std=c99 \
        -o $bname.exe $bname.c $host_list
    ./$bname.exe || {
        echo >&2 "*** Run test of test: exit with $?" ;
        exit 1 ;
    }
    rm -f $bname.exe
}

set_options ()
{
    CFLAGS=
    eval `grep CFLAGS $1`
    cflags="-Wall -W -pipe $CFLAGS"
}

for f in $test_list ; do
    set_src_list $f
    set_options  $f
    test -z $only_avr  &&  run_host $f
    cflags="$cflags -I$srcdir -I$incdir"
    run_simulate $f
done

echo "$0: All OK"
# eof
